{
    This file is part of FloatView.
    Copyright (c) 2003-2008, Tom Verhoeff (TU/e).

    Example illustrating a limitation of floating-point arithmetic.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  *********************************************************************}

program SolveQuadratic;

const
  WIDTH = 30;  { field width for printing }

type
  F = Single;  { the floating-point type for all calculations }
               { candidates: Real, Single, Double, Extended }

procedure WriteSizeOfParamF ( x: F );
  begin
    writeln ( 'SizeOf ( param: F ) = ', SizeOf ( x ) )
  end; { WriteSizeOfParamF }

procedure WriteFrac ( x, y: F );
  var frac: F;
  begin
    if y = 0.0 then begin
      if x = 0.0 then write ( 'NaN' : WIDTH )
      else write ( 'Infinity' : WIDTH )
    end { then }
    else begin
      frac := x / y
    ; write ( frac : WIDTH )
    end { else }
  end; { WriteFrac }

var
  a, b, c: F; { coefficients for quadratic equation (input) }
  two_a, two_c: F; { two_x = 2 * x }
  D, sqrtD: F;  { discriminant sqr(b) - 4ac, and its square root }

begin
  writeln ( 'Solver for quadratic equation  a x^2 + b x + c = 0' )
; writeln ( 'SizeOf ( Floating-Point Type ) = ', SizeOf ( F ) )
{; WriteSizeOfParamF ( 0.0 )}

; write   ( 'Give a, b, c (e.g. 1e-8 1 -1): ' )
; readln ( a, b, c )

; if a = 0.0 then begin
    writeln ( 'a = 0.0, equation is not quadratic' )
  ; halt
  end { if }

; two_a := 2 * a
; two_c := 2 * c
; D := sqr ( b ) - two_a * two_c
; writeln ( 'D      = ', D : WIDTH )
; if D < 0.0 then begin
    writeln ( 'D < 0.0' )
  ; halt
  end { if }
; sqrtD := sqrt ( D )
; writeln ( 'sqrt D = ', sqrtD : WIDTH )

; writeln ( 'Standard a-b-c formula yields:' )
; write   ( 'x_1, 2 = ' )
; WriteFrac ( - b + sqrtD, two_a )
; WriteFrac ( - b - sqrtD, two_a )
; writeln

; writeln ( 'Conjugate a-b-c formula yields:' )
; write   ( 'x_1, 2 = ' )
; WriteFrac ( two_c, - b - sqrtD )
; WriteFrac ( two_c, - b + sqrtD )
; writeln
end.
